@@ -1,7 +1,7 @@
 # Copyright © 1996 Ian Jackson
 # Copyright © 2005 Frank Lichtenheld <frank@lichtenheld.de>
 # Copyright © 2009 Raphaël Hertzog <hertzog@debian.org>
-# Copyright © 2012-2013 Guillem Jover <guillem@debian.org>
+# Copyright © 2012-2015 Guillem Jover <guillem@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,9 +24,8 @@ Dpkg::Changelog::Debian - parse Debian changelogs
 
 =head1 DESCRIPTION
 
-Dpkg::Changelog::Debian parses Debian changelogs as described in the Debian
-policy (version 3.6.2.1 at the time of this writing). See section
-L<"SEE ALSO"> for locations where to find this definition.
+Dpkg::Changelog::Debian parses Debian changelogs as described in
+deb-changelog(5).
 
 The parser tries to ignore most cruft like # or /* */ style comments,
 CVS comments, vim variables, emacs local variables and stuff from
@@ -35,8 +34,6 @@ NOTE: most of these are ignored silently currently, there is no
 parser error issued for them. This should become configurable in the
 future.
 
-=head2 METHODS
-
 =cut
 
 package Dpkg::Changelog::Debian;
@@ -49,22 +46,84 @@ our $VERSION = '1.00';
 use Dpkg::Gettext;
 use Dpkg::File;
 use Dpkg::Changelog qw(:util);
-use parent qw(Dpkg::Changelog);
 use Dpkg::Changelog::Entry::Debian qw(match_header match_trailer);
 
+use parent qw(Dpkg::Changelog);
+
 use constant {
-    FIRST_HEADING => _g('first heading'),
-    NEXT_OR_EOF => _g('next heading or eof'),
-    START_CHANGES => _g('start of change data'),
-    CHANGES_OR_TRAILER => _g('more change data or trailer'),
+    FIRST_HEADING => g_('first heading'),
+    NEXT_OR_EOF => g_('next heading or end of file'),
+    START_CHANGES => g_('start of change data'),
+    CHANGES_OR_TRAILER => g_('more change data or trailer'),
 };
 
+my $ancient_delimiter_re = qr{
+    ^
+    (?: # Ancient GNU style changelog entry with expanded date
+      (?:
+        \w+\s+                          # Day of week (abbreviated)
+        \w+\s+                          # Month name (abbreviated)
+        \d{1,2}                         # Day of month
+        \Q \E
+        \d{1,2}:\d{1,2}:\d{1,2}\s+      # Time
+        [\w\s]*                         # Timezone
+        \d{4}                           # Year
+      )
+      \s+
+      (?:.*)                            # Maintainer name
+      \s+
+      [<\(]
+        (?:.*)                          # Maintainer email
+      [\)>]
+    | # Old GNU style changelog entry with expanded date
+      (?:
+        \w+\s+                          # Day of week (abbreviated)
+        \w+\s+                          # Month name (abbreviated)
+        \d{1,2},?\s*                    # Day of month
+        \d{4}                           # Year
+      )
+      \s+
+      (?:.*)                            # Maintainer name
+      \s+
+      [<\(]
+        (?:.*)                          # Maintainer email
+      [\)>]
+    | # Ancient changelog header w/o key=value options
+      (?:\w[-+0-9a-z.]*)                # Package name
+      \Q \E
+      \(
+        (?:[^\(\) \t]+)                 # Package version
+      \)
+      \;?
+    | # Ancient changelog header
+      (?:[\w.+-]+)                      # Package name
+      [- ]
+      (?:\S+)                           # Package version
+      \ Debian
+      \ (?:\S+)                         # Package revision
+    |
+      Changes\ from\ version\ (?:.*)\ to\ (?:.*):
+    |
+      Changes\ for\ [\w.+-]+-[\w.+-]+:?\s*$
+    |
+      Old\ Changelog:\s*$
+    |
+      (?:\d+:)?
+      \w[\w.+~-]*:?
+      \s*$
+    )
+}xi;
+
+=head1 METHODS
+
 =over 4
 
 =item $c->parse($fh, $description)
 
-Read the filehandle and parse a Debian changelog in it. Returns the number
-of changelog entries that have been parsed with success.
+Read the filehandle and parse a Debian changelog in it. The data in the
+object is reset before parsing new data.
+
+Returns the number of changelog entries that have been parsed with success.
 
 =cut
 
@@ -88,7 +147,7 @@ sub parse {
 	if (match_header($_)) {
 	    unless ($expect eq FIRST_HEADING || $expect eq NEXT_OR_EOF) {
 		$self->parse_error($file, $.,
-		    sprintf(_g('found start of entry where expected %s'),
+		    sprintf(g_('found start of entry where expected %s'),
 		    $expect), "$_");
 	    }
 	    unless ($entry->is_empty) {
@@ -97,7 +156,7 @@ sub parse {
 		last if $self->abort_early();
 	    }
 	    $entry->set_part('header', $_);
-	    foreach my $error ($entry->check_header()) {
+	    foreach my $error ($entry->parse_header()) {
 		$self->parse_error($file, $., $error, $_);
 	    }
 	    $expect= START_CHANGES;
@@ -112,37 +171,30 @@ sub parse {
 	    next; # skip comments, even that's not supported
 	} elsif (m{^/\*.*\*/}o) {
 	    next; # more comments
-	} elsif (m/^(?:\w+\s+\w+\s+\d{1,2} \d{1,2}:\d{1,2}:\d{1,2}\s+[\w\s]*\d{4})\s+(?:.*)\s+[<\(](?:.*)[\)>]/o
-		 || m/^(?:\w+\s+\w+\s+\d{1,2},?\s*\d{4})\s+(?:.*)\s+[<\(](?:.*)[\)>]/o
-		 || m/^(?:\w[-+0-9a-z.]*) \((?:[^\(\) \t]+)\)\;?/io
-		 || m/^(?:[\w.+-]+)[- ](?:\S+) Debian (?:\S+)/io
-		 || m/^Changes from version (?:.*) to (?:.*):/io
-		 || m/^Changes for [\w.+-]+-[\w.+-]+:?\s*$/io
-		 || m/^Old Changelog:\s*$/io
-		 || m/^(?:\d+:)?\w[\w.+~-]*:?\s*$/o) {
+	} elsif (m/$ancient_delimiter_re/) {
 	    # save entries on old changelog format verbatim
 	    # we assume the rest of the file will be in old format once we
 	    # hit it for the first time
 	    $self->set_unparsed_tail("$_\n" . file_slurp($fh));
 	} elsif (m/^\S/) {
-	    $self->parse_error($file, $., _g('badly formatted heading line'), "$_");
+	    $self->parse_error($file, $., g_('badly formatted heading line'), "$_");
 	} elsif (match_trailer($_)) {
 	    unless ($expect eq CHANGES_OR_TRAILER) {
 		$self->parse_error($file, $.,
-		    sprintf(_g('found trailer where expected %s'), $expect), "$_");
+		    sprintf(g_('found trailer where expected %s'), $expect), "$_");
 	    }
 	    $entry->set_part('trailer', $_);
 	    $entry->extend_part('blank_after_changes', [ @blanklines ]);
 	    @blanklines = ();
-	    foreach my $error ($entry->check_trailer()) {
+	    foreach my $error ($entry->parse_trailer()) {
 		$self->parse_error($file, $., $error, $_);
 	    }
 	    $expect = NEXT_OR_EOF;
 	} elsif (m/^ \-\-/) {
-	    $self->parse_error($file, $., _g('badly formatted trailer line'), "$_");
+	    $self->parse_error($file, $., g_('badly formatted trailer line'), "$_");
 	} elsif (m/^\s{2,}(?:\S)/) {
 	    unless ($expect eq START_CHANGES or $expect eq CHANGES_OR_TRAILER) {
-		$self->parse_error($file, $., sprintf(_g('found change data' .
+		$self->parse_error($file, $., sprintf(g_('found change data' .
 		    ' where expected %s'), $expect), "$_");
 		if ($expect eq NEXT_OR_EOF and not $entry->is_empty) {
 		    # lets assume we have missed the actual header line
@@ -164,11 +216,11 @@ sub parse {
 		next;
 	    } elsif ($expect ne CHANGES_OR_TRAILER) {
 		$self->parse_error($file, $.,
-		    sprintf(_g('found blank line where expected %s'), $expect));
+		    sprintf(g_('found blank line where expected %s'), $expect));
 	    }
 	    push @blanklines, $_;
 	} else {
-	    $self->parse_error($file, $., _g('unrecognized line'), "$_");
+	    $self->parse_error($file, $., g_('unrecognized line'), "$_");
 	    unless ($expect eq START_CHANGES or $expect eq CHANGES_OR_TRAILER) {
 		# lets assume change data if we expected it
 		$entry->extend_part('changes', [ @blanklines, $_]);
@@ -179,8 +231,9 @@ sub parse {
     }
 
     unless ($expect eq NEXT_OR_EOF) {
-	$self->parse_error($file, $., sprintf(_g('found eof where expected %s'),
-					      $expect));
+        $self->parse_error($file, $.,
+                           sprintf(g_('found end of file where expected %s'),
+                                   $expect));
     }
     unless ($entry->is_empty) {
 	push @{$self->{data}}, $entry;
@@ -194,22 +247,14 @@ __END__
 
 =back
 
+=head1 CHANGES
+
+=head2 Version 1.00 (dpkg 1.15.6)
+
+Mark the module as public.
+
 =head1 SEE ALSO
 
 Dpkg::Changelog
 
-Description of the Debian changelog format in the Debian policy:
-L<https://www.debian.org/doc/debian-policy/ch-source.html#s-dpkgchangelog>.
-
-=head1 CHANGES
-
-=head2 Version 1.00
-
-Mark the module as public.
-
-=head1 AUTHORS
-
-Frank Lichtenheld, E<lt>frank@lichtenheld.deE<gt>
-Raphaël Hertzog, E<lt>hertzog@debian.orgE<gt>
-
 =cut
