@@ -200,6 +200,7 @@ foreach (keys %{$src_fields}) {
         field_transfer_single($src_fields, $fields);
     }
 }
+$substvars->set_field_substvars($src_fields, 'S');
 
 # Scan binary package
 foreach (keys %{$pkg}) {
@@ -298,8 +299,6 @@ foreach my $field (field_list_pkg_dep()) {
     }
 }
 
-$fields->{'Built-For-Profiles'} = join ' ', get_build_profiles();
-
 for my $f (qw(Package Version Architecture)) {
     error(g_('missing information for output field %s'), $f)
         unless defined $fields->{$f};
@@ -366,63 +365,56 @@ for my $f (keys %remove) {
     delete $fields->{$f};
 }
 
-my $sversion = $fields->{'Version'};
-$sversion =~ s/^\d+://;
-$forcefilename //= sprintf('%s_%s_%s.%s', $fields->{'Package'}, $sversion,
-                           $fields->{'Architecture'}, $pkg_type);
-$forcefilename = $substvars->substvars($forcefilename);
-my $section = $substvars->substvars($fields->{'Section'} || '-');
-my $priority = $substvars->substvars($fields->{'Priority'} || '-');
-
-# Obtain a lock on debian/control to avoid simultaneous updates
-# of debian/files when parallel building is in use
-my $lockfh;
-my $lockfile = 'debian/control';
-$lockfile = $controlfile if not -e $lockfile;
-
-sysopen($lockfh, $lockfile, O_WRONLY)
-    or syserr(g_('cannot write %s'), $lockfile);
-file_lock($lockfh, $lockfile);
-
-my $dist = Dpkg::Dist::Files->new();
-$dist->load($fileslistfile) if -e $fileslistfile;
-
-foreach my $file ($dist->get_files()) {
-    if (defined $file->{package} &&
-        ($file->{package} eq $fields->{'Package'}) &&
-        ($file->{package_type} eq $pkg_type) &&
-        (debarch_eq($file->{arch}, $fields->{'Architecture'}) ||
-         debarch_eq($file->{arch}, 'all'))) {
-        $dist->del_file($file->{filename});
-    }
-}
-
-$dist->add_file($forcefilename, $section, $priority);
-$dist->save("$fileslistfile.new");
-
-rename("$fileslistfile.new", $fileslistfile)
-    or syserr(g_('install new files list file'));
-
-# Release the lock
-close($lockfh) or syserr(g_('cannot close %s'), $lockfile);
-
-my $cf;
-my $fh_output;
-if (!$stdout) {
-    $cf = $outputfile // "$packagebuilddir/DEBIAN/control";
-    open($fh_output, '>', "$cf.new")
-        or syserr(g_("cannot open new output control file '%s'"), "$cf.new");
-} else {
-    $fh_output = \*STDOUT;
-}
-
 $fields->apply_substvars($substvars);
-$fields->output($fh_output);
 
-if (!$stdout) {
-    close($fh_output) or syserr(g_('cannot close %s'), "$cf.new");
-    rename("$cf.new", "$cf")
-        or syserr(g_("cannot install output control file '%s'"), $cf);
+if ($stdout) {
+    $fields->output(\*STDOUT);
+} else {
+    $outputfile //= "$packagebuilddir/DEBIAN/control";
+
+    my $sversion = $fields->{'Version'};
+    $sversion =~ s/^\d+://;
+    $forcefilename //= sprintf('%s_%s_%s.%s', $fields->{'Package'}, $sversion,
+                               $fields->{'Architecture'}, $pkg_type);
+    my $section = $fields->{'Section'} || '-';
+    my $priority = $fields->{'Priority'} || '-';
+
+    # Obtain a lock on debian/control to avoid simultaneous updates
+    # of debian/files when parallel building is in use
+    my $lockfh;
+    my $lockfile = 'debian/control';
+    $lockfile = $controlfile if not -e $lockfile;
+
+    sysopen $lockfh, $lockfile, O_WRONLY
+        or syserr(g_('cannot write %s'), $lockfile);
+    file_lock($lockfh, $lockfile);
+
+    my $dist = Dpkg::Dist::Files->new();
+    $dist->load($fileslistfile) if -e $fileslistfile;
+
+    foreach my $file ($dist->get_files()) {
+        if (defined $file->{package} &&
+            ($file->{package} eq $fields->{'Package'}) &&
+            ($file->{package_type} eq $pkg_type) &&
+            (debarch_eq($file->{arch}, $fields->{'Architecture'}) ||
+             debarch_eq($file->{arch}, 'all'))) {
+            $dist->del_file($file->{filename});
+        }
+    }
+
+    $dist->add_file($forcefilename, $section, $priority);
+    $dist->save("$fileslistfile.new");
+
+    rename "$fileslistfile.new", $fileslistfile
+        or syserr(g_('install new files list file'));
+
+    # Release the lock
+    close $lockfh or syserr(g_('cannot close %s'), $lockfile);
+
+    $fields->save("$outputfile.new");
+
+    rename "$outputfile.new", $outputfile
+        or syserr(g_("cannot install output control file '%s'"), $outputfile);
 }
 
 $substvars->warn_about_unused();
